/**
 *  @file spis.c
 *
 *  @brief I2C Slave implementation for vacuum cleaner interface
 *
 *  $Date$
 *  $Revision$
 *  @copyright P+Z Engineering GmbH - Stuttgart, Germany
 *
 */
 
#include "spis.h"
#include "hardware_config.h"
#include "nrf.h"
#include "nrf_error.h"
#include "nrf_drv_spis.h"
#include "debug.h"

#define DEBUG_MESSAGES


#define SPIS_MAX_DATA_SIZE  200




/* @brief Buffer received by SPI Slave. */
typedef struct __attribute__ ((__packed__)) spis_rx_buffer_s
{
    uint8_t  data[SPIS_MAX_DATA_SIZE];
} spis_rx_buffer_t;

/* @brief Buffer sent by SPI Slave. */
typedef struct __attribute__ ((__packed__)) spis_tx_buffer_s
{
    uint8_t  data[SPIS_MAX_DATA_SIZE];
} spis_tx_buffer_t;




//=================================================================================================
// Private declarations
//=================================================================================================

static const nrf_drv_spis_t       m_spis = NRF_DRV_SPIS_INSTANCE(1);  // The SPI Slave Instance (only SPI1 can be a slave on nRF51)

static volatile spis_rx_buffer_t  m_rx_buffer;
static volatile spis_tx_buffer_t  m_tx_buffer;
static uint32_t                   m_tx_size;

static void spis_event_handler(nrf_drv_spis_event_t event);
static void process_rx_buffer(uint32_t size);

//=================================================================================================
// Implementation
//=================================================================================================

uint32_t init_spis()
{
    uint32_t result = NRF_SUCCESS;
    uint32_t i;

    // Clear buffers
    for (i=0; i<sizeof(m_rx_buffer); i++)
        ((uint8_t*)(&m_rx_buffer))[i] = 0;
    for (i=0; i<sizeof(m_tx_buffer); i++)
        ((uint8_t*)(&m_tx_buffer))[i] = 0;
    m_tx_size = 2;

    const nrf_drv_spis_config_t config =
    {
        .miso_pin     = SPI_SLAVE_MISO_PIN,
        .mosi_pin     = SPI_SLAVE_MOSI_PIN,
        .sck_pin      = SPI_SLAVE_SCK_PIN,
        .csn_pin      = SPI_SLAVE_CSN_PIN,
        .mode         = NRF_DRV_SPIS_MODE_0 ,
        .bit_order    = NRF_DRV_SPIS_BIT_ORDER_MSB_FIRST,
        .csn_pullup   = NRF_GPIO_PIN_NOPULL ,
        .miso_drive   = NRF_GPIO_PIN_S0S1 ,
        .def          = 0xff,
        .orc          = 0xff,
        .irq_priority = APP_IRQ_PRIORITY_LOW
    };

    // Init SPIS
    result = nrf_drv_spis_init(&m_spis, &config, spis_event_handler);

    // If initialization was ok, set first transfer buffer (empty tx buffer)
    if(result == NRF_SUCCESS)
        result = nrf_drv_spis_buffers_set(&m_spis, (uint8_t*)(&m_tx_buffer), m_tx_size, (uint8_t*)(&m_rx_buffer), sizeof(m_rx_buffer));

    return result;
}


static void spis_event_handler(nrf_drv_spis_event_t event)
{
	// !!!! ATTENTION !!!!
	// Bug in event structure: rx_amount and tx_amount values are switched in SDK v10!
	// !!!! ATTENTION !!!!

    uint32_t result;
    uint32_t i;
    if (event.evt_type == NRF_DRV_SPIS_XFER_DONE)
    {
#ifdef DEBUG_MESSAGES
        SEGGER_RTT_printf(0, "\nSPI Slave Transfer done: event.rx_amount = %d bytes, event.tx_amount = %d bytes.\n", event.rx_amount, event.tx_amount);  // Bug in SDK v10!
        SEGGER_RTT_printf(0, "  TX data: ");
        for (i = 0; i < event.tx_amount; i++)  // Bug in SDK v10!
            SEGGER_RTT_printf(0, "%02X ", ((uint8_t*)(&m_tx_buffer))[i]);
        SEGGER_RTT_printf(0, "\b|");            // print separator to mark the buffer end
        for (; i < 16; i++)                    // print up to 32 bytes in buffer
            SEGGER_RTT_printf(0, "%02X ", ((uint8_t*)(&m_tx_buffer))[i]);
        SEGGER_RTT_printf(0, "\n");
        SEGGER_RTT_printf(0, "  RX data: ");
        for (i = 0; i < event.rx_amount; i++)  // Bug in SDK v10!
            SEGGER_RTT_printf(0, "%02X ", ((uint8_t*)(&m_rx_buffer))[i]);
        SEGGER_RTT_printf(0, "\b|");            // print separator to mark the buffer end
        for (; i < 16; i++)                    // print up to 32 bytes in buffer
            SEGGER_RTT_printf(0, "%02X ", ((uint8_t*)(&m_rx_buffer))[i]);
        SEGGER_RTT_printf(0, "\n");
#endif
        // Prepare new buffer for next SPI operation
        //process_rx_buffer(event.tx_amount);  // Bug in SDK v10! tx_amount holds the number ov bytes received by the SPI Slave!

        // For demonstration code: Clear buffers and set TX buffer length to 2 (receive buffer handling has been removed for privacy issues)
        // Clear buffers
        for (i=0; i<sizeof(m_rx_buffer); i++)
            ((uint8_t*)(&m_rx_buffer))[i] = 0;
        for (i=0; i<sizeof(m_tx_buffer); i++)
            ((uint8_t*)(&m_tx_buffer))[i] = 0;
        m_tx_size = 2;

        // Set SPI Slave buffers for next operation
        result = nrf_drv_spis_buffers_set(&m_spis, (uint8_t*)(&m_tx_buffer), m_tx_size, (uint8_t*)(&m_rx_buffer), sizeof(m_rx_buffer));
    }
}

