/**
 *  @file main_display.c
 *
 *  @brief Main file for display target
 *
 *  Startup code, SoftDevice and Bluetooth handling
 *
 *  $Date$
 *  $Revision$
 *  @copyright P+Z Engineering GmbH - Stuttgart, Germany
 *
 */

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf_sdm.h"
#include "ble.h"
#include "softdevice_handler.h"
#include "app_util.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "spis.h"
#include "debug.h"

#define DEBUG_MESSAGES

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */




/* We need the SoftDevice S130 for the Dual Role implementation. The corresponding macro has to be defined in the makefile. */
#ifndef S130
#error "This implementation needs the SoftDevice S130. please make sure that the makefile is configured correctly."
#endif




/**@brief Function for error handling, which is called when an error has occurred.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of error.
 *
 * @param[in] error_code  Error code supplied to the handler.
 * @param[in] line_num    Line number where the handler is called.
 * @param[in] p_file_name Pointer to the file name.
 */
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
#ifdef DEBUG_MESSAGES
	SEGGER_RTT_printf(0, "\n\n====[app_error_handler]====\n* Error code: 0x%08x\n* Line: %d\n* File: %s\n====\nResetting system...\n\n", error_code, line_num, p_file_name);
	nrf_delay_ms(1000);
#endif
    NVIC_SystemReset();
}


/**@brief Function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning Needs to be implemented individually in the end product!
 *
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num     Line number of the failing ASSERT call.
 * @param[in] p_file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(0xDEADBEEF, line_num, p_file_name);
}


/**@brief Function for handling the Application's BLE Stack events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{

}


/**@brief Function for handling the Application's system events.
 *
 * @param[in]   sys_evt   system event.
 */
static void on_sys_evt(uint32_t sys_evt)
{
    switch (sys_evt)
    {
        default:
            break;
    }
}


/**@brief Function for dispatching a BLE stack event to all modules with a BLE stack event handler.
 *
 * @details This function is called from the scheduler in the main loop after a BLE stack event has
 *  been received.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
#ifdef DEBUG_MESSAGES
    //on_ble_evt_debug_messages(p_ble_evt);       // send debug messages
#endif

    on_ble_evt(p_ble_evt);
}


/**@brief Function for dispatching a system event to interested modules.
 *
 * @details This function is called from the System event interrupt handler after a system
 *          event has been received.
 *
 * @param[in]   sys_evt   System stack event.
 */
static void sys_evt_dispatch(uint32_t sys_evt)
{
#ifdef DEBUG_MESSAGES
    on_sys_evt_debug_messages(sys_evt);         // send debug messages
#endif

    on_sys_evt(sys_evt);
}


static void init_softdevice(void)
{
    uint32_t err_code;
    SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_XTAL_20_PPM, false); // Initialize the SoftDevice handler module.
#ifdef S130 // Enable BLE stack
    ble_enable_params_t ble_enable_params; memset(&ble_enable_params, 0, sizeof(ble_enable_params)); // prepare empty ble_enable_params struct
    err_code = sd_ble_enable(&ble_enable_params); APP_ERROR_CHECK(err_code);
#endif
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch); APP_ERROR_CHECK(err_code); // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_sys_evt_handler_set(sys_evt_dispatch); APP_ERROR_CHECK(err_code); // Register with the SoftDevice handler module for SYS events.
}


static void init_bluetooth(void)
{
    uint32_t err_code;

    // Initialize Central Services

    // Initialize Peripheral Service
}


int main(void)
{
    uint32_t result;

    //================================[ Initialization ]================================
#ifdef DEBUG_MESSAGES
    SEGGER_RTT_Init();
    SEGGER_RTT_printf(0, "\n\n====[SPIS Test]==========================\nHWID = %04X\nStarting initialization...\n", (NRF_FICR->CONFIGID & 0xffff));
#endif

    init_softdevice();

    result = init_spis();
    if (result == NRF_SUCCESS) SEGGER_RTT_printf(0, "SPI Slave initialization successful.\n");
    else                       SEGGER_RTT_printf(0, "SPI Slave initialization failed. Error code: 0x%08x\n", result);

    //================================[ Start execution ]================================
#ifdef DEBUG_MESSAGES
    SEGGER_RTT_printf(0, "Starting execution...\n");
#endif


    //================================[ Main loop ]================================
#ifdef DEBUG_MESSAGES
    SEGGER_RTT_printf(0, "Starting main loop...\n");
#endif

    for (;;)
    {
        nrf_delay_ms(100);
    }
}

